import os
from typing import Tuple, TypedDict, List

from matplotlib import pyplot as plt
import seaborn as sns
from matplotlib.patches import Patch


class Margin(TypedDict):
    left: float
    top: float
    right: float
    bottom: float


class Spacing(TypedDict):
    vertical: float
    horizontal: float


# Config about general parameters
class Config:
    InputPath = "input"
    IgnoreFolder = [
        "All",
    ]

    DataOutputPath = "out/data"
    JaspOutputPath = "out/jasp"
    OtherContentOutputPath = "out/other"
    PaperContentOutputPath = "out/paper"


# Config for values for the figures
class FiguresConfig:
    Dpi = 200
    FigureFactor = 5

    FullColumn = 7.032 * FigureFactor
    HalfColumn = 3.349 * FigureFactor
    PageHeight = 10 * FigureFactor

    VisFormats = [
        "jpg",
        "pdf",
    ]
    SnsStyle = "whitegrid"

    RC_FONTS = {
        "font": 29,  # controls default text sizes
        "axes": {
            "title": 28,  # fontsize of the axes title
            "label": 29  # fontsize of the x and y labels
        },
        "xtick": 31,  # fontsize of the tick labels
        "ytick": 31,  # fontsize of the tick labels
        "legend": 31,  # legend fontsize
        "figure": 28,  # fontsize of the figure title
    }

    Palettes = {
        "Adaptation": {
            "Adapted": "#4daf4a",
            "Non-Adapted": "#984ea3"
        },
        "ChartType": {
            "Bar": "#ff7f00",
            "Scatter": "#377eb8",
        }
    }

    AxisLabels = {
        "Time": "TCT in s",
        "Accuracy": "TA",
        "Dependability": "Dependability",
        "Usefulness": "Usefulness",
        "Intuitive Use": "Intuitive Use",
        "VL-Bar": "VL Score (Bar Chart)",
        "VL-Scatter": "VL Score (Scatter Plot)",
        "VL": "VL Score"
    }

    BorderWidth = 5
    RangeLimitFactor = 0.1
    Scatter = True

    @staticmethod
    def calc_margin_complete(fig_size: Tuple[float, float], absolute_margin: float) -> Margin:
        return {
            "left": absolute_margin / fig_size[0],
            "top": 1. - (absolute_margin / fig_size[1]),
            "right": 1. - (absolute_margin / fig_size[0]),
            "bottom": absolute_margin / fig_size[1]
        }

    @staticmethod
    def calc_margin_single(fig_size: Tuple[float, float], absolute_margin:  Tuple[float, float, float, float]) -> Margin:
        return {
            "left": absolute_margin[0] / fig_size[0],
            "top": 1. - (absolute_margin[1] / fig_size[1]),
            "right": 1. - (absolute_margin[2] / fig_size[0]),
            "bottom": absolute_margin[3] / fig_size[1]
        }

    @staticmethod
    def calc_spacing_complete(fig_size: Tuple[float, float], absolute_spacing: float) -> Spacing:
        return {
            "vertical": absolute_spacing / fig_size[0],
            "horizontal": absolute_spacing / fig_size[1]
        }

    @staticmethod
    def calc_spacing_single(fig_size: Tuple[float, float], absolute_spacing: Tuple[float, float]) -> Spacing:
        return {
            "vertical": absolute_spacing[0] / fig_size[0],
            "horizontal": absolute_spacing[1] / fig_size[1]
        }

    @staticmethod
    def set_rc_plot_values():
        # controls default text sizes
        plt.rc('font', size=FiguresConfig.RC_FONTS["font"])
        # fontsize of the axes title
        plt.rc('axes', titlesize=FiguresConfig.RC_FONTS["axes"]["title"])
        # fontsize of the x and y labels
        plt.rc('axes', labelsize=FiguresConfig.RC_FONTS["axes"]["label"], labelweight="bold")
        # fontsize of the tick labels
        plt.rc('xtick', labelsize=FiguresConfig.RC_FONTS["xtick"])
        # fontsize of the tick labels
        plt.rc('ytick', labelsize=FiguresConfig.RC_FONTS["ytick"])
        # legend fontsize
        plt.rc('legend', fontsize=FiguresConfig.RC_FONTS["legend"])
        # fontsize of the figure title
        plt.rc('figure', titlesize=FiguresConfig.RC_FONTS["figure"])

    @staticmethod
    def set_custom_legend(keys: List[str], colors: List[str], fig: plt.Figure = None, ax: plt.Axes = None,
                          title: str = None, label_on_fig: bool = True, loc: str = "upper center", bbox_to_anchor_y: float = 1.01):

        obj = fig if label_on_fig else ax
        custom_lines = [Patch(facecolor="#ffffff", edgecolor="#ffffff")] + [Patch(facecolor=c, edgecolor=c) for c in
                                                                            colors]
        leg = obj.legend(
            custom_lines,
            [title + ":"] + keys,
            loc=loc,
            bbox_to_anchor=(0.5, bbox_to_anchor_y),
            ncol=len(keys) + 1,
            framealpha=0.95,
            edgecolor="#000000",
        )
        for vpack in leg._legend_handle_box.get_children()[:1]:
            for hpack in vpack.get_children():
                hpack.get_children()[0].set_width(0)


# Config for all the data related things.
class DataConfig:
    FixedValues = {
        "Time": [
            20,
            30,
            40,
            50,
            60,
            70
        ],
        "Accuracy": [
            0,
            1
        ],
        "Dependability": [
            -3, -2, -1, 0, 1, 2, 3
        ],
        "Usefulness": [
            -3, -2, -1, 0, 1, 2, 3
        ],
        "Intuitive Use": [
            -3, -2, -1, 0, 1, 2, 3
        ],
        "VL": [
            -0.4,
            0,
            .5,
            1
        ],
        "VL-Scores": [
            -1.5,
            -1,
            -0.5,
            0,
            0.5,
            1
        ],
        "VL-Bar": [
            -1,
            0,
            1
        ],
        "VL-Scatter": [
            -1.72,
            -1,
            0,
            1
        ],
        "KDE-VL": [
            0,
            .2,
            .4,
            .6,
            .8,
            1
        ],
        "Hist-VL": [
            0,
            4,
            8,
            12,
            16,
            20
        ]
    }

    TaskMap = {
        "Bar": [1, 2, 3, 4, 5, 11, 12, 13, 14, 15],
        "Scatter": [6, 7, 8, 9, 10, 16, 17, 18, 19, 20],
        "Non-Adapted": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
        "Adapted": [11, 12, 13, 14, 15, 16, 17, 18, 19, 20],
        "Bar-NonAdapted": [1, 2, 3, 4, 5],
        "Scatter-NonAdapted": [6, 7, 8, 9, 10],
        "Bar-Adapted": [11, 12, 13, 14, 15],
        "Scatter-Adapted": [16, 17, 18, 19, 20],
    }

    @staticmethod
    def calc_fixed_value_limits(fixed_value: str) -> Tuple[float, float]:
        if fixed_value not in DataConfig.FixedValues:
            return (None, None)

        value_range = DataConfig.FixedValues[fixed_value][-1] - DataConfig.FixedValues[fixed_value][0]
        return (
            DataConfig.FixedValues[fixed_value][0] - value_range * FiguresConfig.RangeLimitFactor,
            DataConfig.FixedValues[fixed_value][-1] + value_range * FiguresConfig.RangeLimitFactor,
        )


os.makedirs(Config.DataOutputPath, exist_ok=True)
os.makedirs(Config.JaspOutputPath, exist_ok=True)
os.makedirs(Config.PaperContentOutputPath, exist_ok=True)
os.makedirs(Config.OtherContentOutputPath, exist_ok=True)
